/*
 * Copyright 2019 Immutables Authors and Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.immutables.criteria.personmodel;

import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import org.immutables.criteria.Criteria;
import org.immutables.criteria.repository.reactive.ReactiveWatchable;
import org.immutables.criteria.repository.sync.SyncReadable;
import org.immutables.criteria.repository.sync.SyncWritable;
import org.immutables.value.Value;

import java.io.Serializable;
import java.time.LocalDate;
import java.util.List;
import java.util.Optional;
import java.util.Set;

/**
 * Test model for Criteria API.
 *
 * Not too simple and not too complicated.
 */
@Value.Immutable
@Criteria
@Criteria.Repository(facets = {SyncReadable.class, SyncWritable.class, ReactiveWatchable.class})
@JsonSerialize(as = ImmutablePerson.class)
@JsonDeserialize(as = ImmutablePerson.class)
public interface Person extends Serializable // serializable needed for Geode temporarily
{

  @Criteria.Id
  String id();

  boolean isActive();

  String fullName();

  Optional<String> nickName();

  LocalDate dateOfBirth();

  int age();

  Optional<Address> address();

  Set<String> interests();

  Optional<Friend> bestFriend();

  List<Pet> pets();

}
